;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xe93fb957e518f7048dd3febb13ce2d43 . "./code.cg2js/cg-acache.fasl")
(#x8ef69bcce21e4757157e47324725af91 . "./code.cg2js/cg-base.fasl")
(#xdc41310a0b9045b492ebd7afdcfa3185 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x7cd820eb373866996dba1e80b7a3c7f8 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x052c23b6a48e39167f2d0dc7ce79312e . "./code.cg2js/cg-bitmap-stream.fasl")
(#x057cd3b544133dbee3c9ac8fe8bf4cf2 . "./code.cg2js/cg-button.fasl")
(#x83a93ae063f6da03ee56d0103232fbe9 . "./code.cg2js/cg-calendar.fasl")
(#x2221baeab0d4c6f5290c5ce4987fbdd3 . "./code.cg2js/cg-caret.fasl")
(#x305d09c812406a27062bfe731ea80bdc . "./code.cg2js/cg-chart-or-plot.fasl")
(#x4233525fe93cc0d2db5556a8482022d8 . "./code.cg2js/cg-chart-widget.fasl")
(#xbaca2fc642e9db9a09e70a351d27733d . "./code.cg2js/cg-check-box.fasl")
(#x794cf9e8a409dc4d598ed46aaf8b0a1c . "./code.cg2js/cg-choice-list.fasl")
(#x00c55fbffb918d04bbc1dc2e2cc804a7 . "./code.cg2js/cg-choose-printer.fasl")
(#x38cd69021ca7907e5c2c75ad0af5c5f7 . "./code.cg2js/cg-class-grid.fasl")
(#x60723634569ae9c65a9dbe898024dd2b . "./code.cg2js/cg-class-slot-grid.fasl")
(#x45259f916d2a3e5b7716b60b2e62c955 . "./code.cg2js/cg-class-support.fasl")
(#x835ecc08d3530c372161b1b895b1e83d . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x757e77a9a8927732144b1bf9875dae2d . "./code.cg2js/cg-clipboard-stack.fasl")
(#x536b419f375c0c6d732798d1ada45de0 . "./code.cg2js/cg-clipboard.fasl")
(#xfba8b14b1673fe3751710822f82d18f4 . "./code.cg2js/cg-color-dialog.fasl")
(#xb8b387975b9cdeef9f64a8f3418674ac . "./code.cg2js/cg-combo-box.fasl")
(#xfbfc54843d42494d8ab30cdd60f71c38 . "./code.cg2js/cg-common-control.fasl")
(#xe2c159ed4bbc61095a67a43a76c5a2aa . "./code.cg2js/cg-comtab.fasl")
(#x56725ec7b080b3dde7e0d4be3690f4f6 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x9472450a3456a60c16794311025dbb70 . "./code.cg2js/cg-curve.fasl")
(#x8015510fb1ae7b108d145f103f14e3bb . "./code.cg2js/cg-dde-utils.fasl")
(#x0c75db51d7efc793131ee6d0c9f94b4d . "./code.cg2js/cg-dev.fasl")
(#x0d0d5c08f8b6fe553e20870fddec699d . "./code.cg2js/cg-dialog-item.fasl")
(#x3e221f73c5b90f358d51df58172a800a . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xfa39a88e635835bd527cb7c0b255b13e . "./code.cg2js/cg-directory-dialog.fasl")
(#xb9748df9821a6037bef98724bbcc9314 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x14f51f02dc3b023cd5ec1cb46a022f1c . "./code.cg2js/cg-drag-and-drop.fasl")
(#xb0f1d514ae00a31e76486091c7e248e6 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x327a99177987259a80a7f1a83a135d83 . "./code.cg2js/cg-drawable.fasl")
(#x13639ed6ba864447eac181ecb719b18d . "./code.cg2js/cg-dropping-outline.fasl")
(#xb09ec8dc4b2f981555275528f12aa7cc . "./code.cg2js/cg-edit-in-place.fasl")
(#xd1a1b92e41f98676dbe53eeb2aed6343 . "./code.cg2js/cg-editable-text.fasl")
(#xc5a055e680a0777126e3a23825858c23 . "./code.cg2js/cg-file-dialog.fasl")
(#x6380fca0a55563074afc91dabaf4876d . "./code.cg2js/cg-fill-texture.fasl")
(#x2e5471c29aba57586326f5646ed76300 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x8d2f6e880e4ac9f1bb2241ee06f1d739 . "./code.cg2js/cg-font-dialog.fasl")
(#xb5224620ff20596aa04b11a1af1cd010 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xdc4f3e48e9df5f5faf0227a3c6f63960 . "./code.cg2js/cg-get-pixmap.fasl")
(#x5d534f8e1e2650842a709ba947d8025d . "./code.cg2js/cg-get-position.fasl")
(#xaee364380801c648411fd1bd891fd095 . "./code.cg2js/cg-graphics-context.fasl")
(#xc32e35b2f08db7f9649922f206a3b2a8 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xe8c8620880d6a2326cc771b6dca97d53 . "./code.cg2js/cg-grid-widget.fasl")
(#x1e8a2bfc9680a161d28ca132e1052efc . "./code.cg2js/cg-group-box.fasl")
(#xf1a6430432ebb947998565824388ebbc . "./code.cg2js/cg-header-control.fasl")
(#xd7049ec8d875259fbbdedaeea76a074c . "./code.cg2js/cg-hotspot.fasl")
(#x73435942a1dfd43c46f0591ed56cee66 . "./code.cg2js/cg-html-dialog.fasl")
(#x26235b47cf3498af7fb93d1fd2cb80b0 . "./code.cg2js/cg-html-widget.fasl")
(#x42552247b2df5274f029228646cbbf37 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xf9b62584cfc0893d3f7caee2fde7c011 . "./code.cg2js/cg-icon.fasl")
(#x50dbabf24f613b4452759259dc6e74af . "./code.cg2js/cg-ie.fasl")
(#x524e99c06b6274398a66e5481635d033 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xd936535f0fd040d7800fcbcc3ee4e339 . "./code.cg2js/cg-intersect.fasl")
(#xd0495fe061bf9cb2baa6ec0b55a07d5f . "./code.cg2js/cg-item-list.fasl")
(#x91cc81f5c961a814a27793e61a80c4db . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xac7264cba4ce44b8b35e78f6d36e4773 . "./code.cg2js/cg-lamp.fasl")
(#xadd3166734d43d6ec4e8455ba9365f25 . "./code.cg2js/cg-layout.fasl")
(#x1ceed1faa1461764b8faf484af4c9c2f . "./code.cg2js/cg-lettered-menu.fasl")
(#x3b0d45c0bb2f8767808b7cec47ebebd3 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x99833502605830b72c990e81132db334 . "./code.cg2js/cg-lisp-text.fasl")
(#x768e2400e3370257cdb1928e01b9f6b6 . "./code.cg2js/cg-lisp-widget.fasl")
(#x8f2c75fb2630c45ee48a3eb22208fc5e . "./code.cg2js/cg-list-view.fasl")
(#xf8db0a845cfb7770ae4cd09287849789 . "./code.cg2js/cg-mci.fasl")
(#x48f8f25139e08f20cdd1f544932f64c4 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x629fc78f4b8ead79a01273d3502331c7 . "./code.cg2js/cg-menu.fasl")
(#x560ef00d3db96ba11097c815798d0c60 . "./code.cg2js/cg-message-dialog.fasl")
(#xfbc7815c3c786f196552226289f5152f . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xba1ba41f82c5b2b3c8508174c7be6bd7 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x8ef70cfbe4a69bde7ff5ba8d8097650d . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xaa80e5d2a5936eba42c4328703c4f6fc . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x6ad947e5008dd53c45bd495a72f8a3e4 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x6cd0d32fe5a4026ebca1c28041089872 . "./code.cg2js/cg-nodes.fasl")
(#x4049cb8797484498d8a7aab0f3c4becc . "./code.cg2js/cg-object-editor-layout.fasl")
(#x7d29fef51016209954d168ab3999fedb . "./code.cg2js/cg-object-editor.fasl")
(#x68058219ba9596b2abc016dc122071c5 . "./code.cg2js/cg-ocx.fasl")
(#xe82d69eb5523c267954450b04b324b73 . "./code.cg2js/cg-os-widget.fasl")
(#x5edd04ffb50e62eb8bda16e1f51fa8d7 . "./code.cg2js/cg-os-window.fasl")
(#xa48e8f9778fe56c3467246927510768e . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xf127b7c08fddf50e4e9370729befb94e . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xf8d6c00046fb145eac0d550a81a8715b . "./code.cg2js/cg-outline.fasl")
(#x661f4b7b39288140c4d42904ce15d98c . "./code.cg2js/cg-palette.fasl")
(#x7e3dba63a8521e9c41e7f2b209f41d4b . "./code.cg2js/cg-paren-matching.fasl")
(#x58bde64fdeccc35d4d94877c300facfb . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xae771dce56fe04c85355df57cae026e8 . "./code.cg2js/cg-picture-widget.fasl")
(#xeed600318368d0d55dac0c56f8971212 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x9e65e427641aa958cdd60964ca0f8156 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x990792c4688e502cbbe2be527624b21d . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x79022264e112c29feba3cb1a78085acf . "./code.cg2js/cg-pixmap-widget.fasl")
(#x39b38ce94ae97b1a43011944a55758f6 . "./code.cg2js/cg-pixmap.fasl")
(#x40b85cf9ca4a343f5083af93d17b3f30 . "./code.cg2js/cg-player.fasl")
(#xdb8f9d9f93b0b54bd95606401e14d223 . "./code.cg2js/cg-plot-widget.fasl")
(#x0beb1693daa1415ef3dd660b3f9866c6 . "./code.cg2js/cg-printing.fasl")
(#x4fc76ffab34796413d1a664275b6f51c . "./code.cg2js/cg-progress-indicator.fasl")
(#x88323a8ba84ceec65e9f2c7b2e508f2d . "./code.cg2js/cg-project-window.fasl")
(#x6fd077f2bbdfbb145a2890d00977e370 . "./code.cg2js/cg-property.fasl")
(#x637add6f921c1d66f9cd2d148078f7d2 . "./code.cg2js/cg-radio-button.fasl")
(#x04bab5ff1ab794b192f1f2fb4eb6dec7 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x39e88811d32ad4c6c4495c3b754b409e . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x76f0adfe6fa3cae68c5bdcd6faaa085e . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x5ca193f32664ceb5e2cc1b833e08ed46 . "./code.cg2js/cg-rich-edit.fasl")
(#xff6f2e331050545f227b527bc7c900b5 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xc1e7fb44b13c3bf8d1cf168b3efdf0da . "./code.cg2js/cg-scaling-stream.fasl")
(#x7682096eabcded2aaf74bcc1a0e22a41 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xe1347a2a75861c2ed1ba250c3bc0e747 . "./code.cg2js/cg-scroll-bar.fasl")
(#x49736c8a2d2f5d23000e577e7bf42f25 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xca15c8ff60c946b67e0a84e26be34be9 . "./code.cg2js/cg-selected-object.fasl")
(#x2fb83e803c0e22d1a8d980fa1374c6e0 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x5c09e29b9de5ac690cc76e1758983577 . "./code.cg2js/cg-split-bar.fasl")
(#xaa8dc51104fcaa9edf831ac7bc285c9a . "./code.cg2js/cg-static-text.fasl")
(#xbfbb3051daeb6e36163c16dd3cccf402 . "./code.cg2js/cg-status-bar.fasl")
(#x6084a7cd9725ca5d7240b08f1e920a67 . "./code.cg2js/cg-string-dialog.fasl")
(#xe937ef4770c1f3ef4ff7d929aac1d2c3 . "./code.cg2js/cg-tab-control.fasl")
(#x4a81a72b23ac4fa906cc5fc9e6ee03df . "./code.cg2js/cg-template-string.fasl")
(#xa87d1bd83f78bf012d549b95c8d00c25 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xf9726997d2da0c956dc7da9c41393e19 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x1d24c7290ae64a4a29c864d725feecf6 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x5eb330d3bc0328ddd259bddd9e170747 . "./code.cg2js/cg-text-or-combo.fasl")
(#x2da2c7ee5c52dea2675b10ebeb1dbc05 . "./code.cg2js/cg-text-widget.fasl")
(#xf5827a5648ebd28e07a929804a967bc9 . "./code.cg2js/cg-timer.fasl")
(#x95759450f5bed3be5ea4c6dcec838f9c . "./code.cg2js/cg-toggling-widget.fasl")
(#xba2c072bc6c766f59ab80f8c0c2d3b26 . "./code.cg2js/cg-toolbar.fasl")
(#x96e00c6506e1a8d03fc5fac09f8b6d1b . "./code.cg2js/cg-tooltip.fasl")
(#x4f322e03e924ad4f2d13742e2e77d54e . "./code.cg2js/cg-touch.fasl")
(#x7978c2b2093669da056960a5ccea76fa . "./code.cg2js/cg-trackbar.fasl")
(#x15af26f6aabe20380623794c3b00967a . "./code.cg2js/cg-tray.fasl")
(#x458de71dc927340ce2009371dfc32ac4 . "./code.cg2js/cg-up-down-control.fasl")
(#x573074eb4c082f14c4fb4c47bb9a790a . "./code.cg2js/cg-utility-dialog.fasl")
(#x2ea406a9562d8db179e1f91d46962b21 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x75588529b78a3a098960174591383e3c . "./code.cg2js/cg-web-browser.fasl")
(#xb34346c3a688fd09c66bb5f55d88deba . "./code.cg2js/cg-wrap-string.fasl")
(#x05882d288dd1ebfbfe9db13f15482bdb . "./code.cg2js/cg-yes-no-list.fasl")
(#x33637bedb916af04105c9f1028b91f8a . "./code.cg2js/cg.fasl")
(#xd1d275d7c0eec1b93df8cfab708a6457 . "./code.cg2js/cg-yes-no-string.fasl")
(#x513aad63061d2014e93d21261ba64cf9 . "./code.idejs/ide-acache.fasl")
(#x166e48f49cda73862c4c675bdd4177a5 . "./code.idejs/ide-apropos.fasl")
(#x18de40f0fe7cc6a1fa77ec20fca70152 . "./code.idejs/ide-base.fasl")
(#x82d2cd63a83a4f7d498213ef81fabc3d . "./code.idejs/ide-class-browser.fasl")
(#xe47ecd83f2eb6eab4c26fb5c777149fa . "./code.idejs/ide-class-grapher.fasl")
(#x5f3394ef20a8586bc0acc5dedb559939 . "./code.idejs/ide-clipboard.fasl")
(#xcc1b86ed6587367c5ab2fb1025dc135c . "./code.idejs/ide-dev.fasl")
(#xcc2b731125b9ada68c06b7cee1c4e3ed . "./code.idejs/ide-editor.fasl")
(#x2250d8ab07daaa38da9cc3f12f695692 . "./code.idejs/ide-find-definitions.fasl")
(#xa9c6405227d670de6945d93687ee5327 . "./code.idejs/ide-find-in-files.fasl")
(#x347529fb44e54a118406ed759099befa . "./code.idejs/ide-form.fasl")
(#xf246c81529eee359c198f75e19ac1990 . "./code.idejs/ide-grapher.fasl")
(#x215c89c97fa222ec6ef35934614a44fb . "./code.idejs/ide-inspector.fasl")
(#xabb11424db1443053e69f41697d74aec . "./code.idejs/ide-intro.fasl")
(#xdadf67b4dccbd81e8894375ec7bb3532 . "./code.idejs/ide-keyshort.fasl")
(#x1ece0939cbd95c9a88e9250fb99e482a . "./code.idejs/ide-navigator.fasl")
(#x83a9b21fddcb8d23d835c9af2847dc15 . "./code.idejs/ide-package-list.fasl")
(#xeb870c433a0e30197b944559a57415a1 . "./code.idejs/ide-patch-dialog.fasl")
(#xea17422826b1dc4064cabf5986ba6885 . "./code.idejs/ide-process-browser.fasl")
(#xf020a13e3a08f34835c2f0382939f39c . "./code.idejs/ide-project.fasl")
(#x00c41e9a84ac692a14b0767320fa2561 . "./code.idejs/ide-trace.fasl")
(#x6f2cf62ca02d3d3ca5e2545554c5ce16 . "./code.idejs/ide-window-list.fasl")
(#xed29df4857604ba6be040b2b05ff026f . "./code.idejs/ide.fasl")
(#xf82fd41d79afb1da19d111da8345005f . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
